//Ce programme utilise les bibliothèques IRremote, Stepper et LiquidCrystal_I2C
//Daniel Buret 20/03/2023
//Tortue-Robot avec 2 moteurs pas à pas, commande IR et écran LCD 
//Télécommande Joy-it (vendu par Go Tronic)

#include "IRremote.h"
#include <Stepper.h>
#include <LiquidCrystal_I2C.h>

//Contantes et Variables globales
const int pasParTour = 2048; // nombre de pas par tour du moteur
const double diamRoue=6.7; // diamètre (en cm) des roues motrices
const double distIntRoue=15.58; //distance (en cm) entre les deux roues (milieu à milieu)
const double pi=3.14159265;
int receiver = 12; // Signal Pin of IR receiver to Arduino Digital Pin 12
String commande=""; //variable pour mémoriser les commandes d'une procédure
String comdirecte=""; //variable pour mémoriser une commande directe 
String comparam=""; //variable pour mémorisée 4 caractères (commande avec paramètre)
double peri=pi*diamRoue; //distance pour un tour de roue soit le périmètre de la roue
double disang=pi*distIntRoue/360; // distance à parcourir pour 1 degré
bool prog=false; // pour passer en mode procédure
bool execp=false; //programme non exécuté"
bool execdc=false;//commande directe non exécutée
bool err=false;//pour erreur dans une commande

//Initialisation Moteurs et écran LCD
//l'écran LCD est relié aux pins A4 et A5 (SDA en A4 et SCL en A5)
LiquidCrystal_I2C lcd(0x3F,16,2); //Initialise l'écran LCD Série(adresse : 0x3F ou 0x27)
Stepper MoteurPasAPasD(pasParTour, 8, 10, 9, 11);// initialise la librairie du moteur droit avec les pins de 8 à 11
Stepper MoteurPasAPasG(pasParTour, 4, 6, 5, 7);//initialise la librire du moteur gauche avec les pins 2 à 5 
 
//Fonctions
void ExeProg(String chainecom){
  String pascom="";String carac="";int p1=0;int p2=0;int car=0;int pt=0;
  pt=chainecom.length();
  chainecom=chainecom+"*";
  while(p1<chainecom.length()){
    p2=p1+2;
    carac=chainecom.substring(p2,p2+4);
    car = carac.toInt();
    //3 chiffres maxi pour le nombre de pas
    //ou pour la valeur de l'angle (donc inférieur à 1000)
    if(car>999){
      AffichageL2("param trop grand", true);
      err=true;
      p1=chainecom.length();
    }
    else{
      carac=chainecom.substring(p2,p2+3);
      car = carac.toInt();
      if(car<100){
        if(car<10){
          pascom = chainecom.substring(p1,p2+1);
          p1=p2+1;
        }
        else{
          pascom = chainecom.substring(p1,p2+2);
          p1=p2+2;
        } 
      }
      else{
        pascom = chainecom.substring(p1,p2+3);
        p1=p2+3;
      }
      if(pascom.substring(0,2)=="RP"){
        pt=chainecom.lastIndexOf('F');
        p1=TraitRepete(pascom,chainecom,p1,pt);
      }
      else{
        ExeCom(pascom); //Exécution de pascom
      }
    }
  }
}

int TraitRepete(String pascom, String chainecom, int p, int pt){
  int nbr=0; String pascomrep;
  nbr=pascom.substring(2).toInt();
  pascomrep=chainecom.substring(p,pt);
  for(int i=1; i<nbr+1; i++){
    ExeProg(pascomrep);
  }
  return pt+2;
}

void ExeCom(String comdir){
  String com=""; String numcom=""; int ncom;
  AffichageL2(comdir, true);
  com=comdir.substring(0,2);
  numcom=comdir.substring(2);
  ncom = numcom.toInt();
  Moteurs(com,ncom); //mise en marche des moteurs, commande : AV, RE, TD, TG, pas : distance ou angle
}

void Moteurs(String commande, int pasang){
  double nbtour=0;
  if(commande=="AV"){
    nbtour=pasang/peri;
    for(int i=0; i<nbtour*pasParTour; i++){
      MoteurPasAPasD.step(1);
      MoteurPasAPasG.step(1);
    }
  }
  if(commande=="RE"){
    nbtour=pasang/peri;
    for(int j=0; j<nbtour*pasParTour; j++){
      MoteurPasAPasD.step(-1);
      MoteurPasAPasG.step(-1);
    }
  }  
  if(commande=="TD"){
    while(pasang>360){
      pasang=pasang-360;
    }
    nbtour=(pasang*disang)/peri;
    for(int m=0; m<nbtour*pasParTour; m++){
      MoteurPasAPasD.step(1);
      MoteurPasAPasG.step(-1); 
    }
  }
  if(commande=="TG"){
    while(pasang>100){
      pasang=pasang-360;
    }
    nbtour=(pasang*disang)/peri;
    for(int n=0; n<nbtour*pasParTour; n++){
      MoteurPasAPasD.step(-1);
      MoteurPasAPasG.step(1); 
    }
  }
}

String EffaceDerCar(String compar){
  String comp="";
  if(compar.length()>2){
   comp=compar.substring(0, compar.length()-1); 
  }
  return comp;
}

void DecodageIR(){
  switch(IrReceiver.decodedIRData.command){
    case 7: if(prog){commande=commande + comparam;}else {commande="";}; comparam="TG"; break;
    case 9: if(prog){commande=commande + comparam;}else {commande="";}; comparam="TD"; break;
    case 25: if(prog){commande=commande + comparam;}else {commande="";}; comparam="RE"; break;
    case 64: if(prog){commande=commande + comparam;}else {commande="";}; comparam="AV"; break;
    case 69: if(prog){commande=commande + comparam;}else {commande="";}; comparam="RP"; break;
    case 71: if(prog){commande=commande + comparam;}else {commande="";}; comparam="FR"; break;
    case 22: comparam=comparam+"0"; break;
    case 12: comparam=comparam+"1"; break;
    case 24: comparam=comparam+"2"; break;
    case 94: comparam=comparam+"3"; break;
    case 8: comparam=comparam+"4"; break;
    case 28: comparam=comparam+"5"; break;
    case 90: comparam=comparam+"6"; break;
    case 66: comparam=comparam+"7"; break;
    case 82: comparam=comparam+"8"; break;
    case 74: comparam=comparam+"9"; break;
    case 21:
      if(prog & comparam !=""){
        commande=commande+comparam; ExeProg(commande);
        if(!err){AffichageMem(commande);};comparam="";
        comdirecte="";execp=true;execdc=false;prog=false;
      }
      if(!prog & comparam !=""){
        comdirecte=comparam;ExeCom(comdirecte);AffichageMem(comdirecte);
        comparam="";commande="";execdc=true;execp=false;
      }
      break;
    case 67: prog=true; execp=false; execdc=false; comparam=""; commande=""; break;
    case 68: comparam=EffaceDerCar(comparam); break;
    case 13:
      if(execp){ExeProg(commande);AffichageMem(commande);}
      if(execdc){ExeCom(comdirecte);AffichageMem(comdirecte);}
      break;
  }
  //Affichage pour un programme
  if(prog){AffichageL1("Programme :");}
  if(prog & commande==""){AffichageL2(comparam, false);}
  if(prog & commande!=""){AffichageProg(commande+comparam);}
  //Affichage pour une commande directe
  if(!prog & !execdc & comparam!=""){AffichageL1("Commande :");}
  if(!prog & !execp & execdc & comparam!="" & commande==""){AffichageL1("Commande :");}
  if(!prog & !execp & execdc & comparam!="" & commande==""){AffichageL2(comparam,false);} 
  if(!prog & !execp & !execdc & commande==""){AffichageL2(comparam, false);}
  if(!prog & execp & comdirecte=="" & comparam!=""){AffichageL2(comparam,false);}
  
}

void AffichageProg(String p){
  int q=p.length()/16;
  if(p.length()>q*16){
    AffichageL1("Programme :");
    AffichageL2(p.substring(16*q), false);
  }
  else{
    AffichageL2(p, false);
  }
}

void AffichageL1(String t1){
  lcd.clear();
  lcd.setCursor(0, 0); //colonne 0, ligne 0
  lcd.print(t1); 
}

void AffichageL2(String t2,bool efface){
  if(efface){lcd.clear();}
  lcd.setCursor(0,1); //colonne 0, ligne 1
  lcd.print(t2);
}

void AffichageMem(String t3){
  lcd.clear();
  lcd.setCursor(0, 0); lcd.print("En memoire :");
  lcd.setCursor(0, 1); lcd.print(t3); 
}

void setup(){
  IrReceiver.begin(receiver, ENABLE_LED_FEEDBACK);
  MoteurPasAPasD.setSpeed(9);// règle la vitesse à 9 (maxi entre 10 et 15)
  MoteurPasAPasG.setSpeed(9);
  //Affichage écran LCD
  lcd.init(); // Initialisation du driver de l'écran
  lcd.backlight(); // Active le rétroéclairage
  lcd.setCursor(0, 0); //colonne 0, ligne 0
  lcd.print("Tortue mobile :");
  lcd.setCursor(0, 1); //colonne 0, ligne 1
  lcd.print("bonjour !");
}

void loop(){
  if (IrReceiver.decode()) {
      IrReceiver.resume(); // rend actif la prochaine réception de signal
      DecodageIR();
  }
  delay(2000); // Pour ne pas répéter immédiatement (évite les doublons)
}
